# Author: Kapil Thangavelu <kvthan@wm.edu>
# License: GPL
# CVS: $Id: ExtIndex.py,v 1.3 2001/09/29 23:33:50 ender Exp $

 

from Products.PluginIndexes import PluggableIndex 
from Products.PluginIndexes.common.util import parseIndexRequest

from Globals import Persistent, DTMLFile

from Acquisition import Implicit
from OFS.History import Historical
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

from BTrees.IOBTree import IOBTree
from BTrees.OIBTree import OIBTree
from BTrees.IIBTree import IISet

from SwishSearch import swish_search

import re


_marker = []

addExtIndexForm = DTMLFile('dtml/manage_addExtIndexForm', globals())


def addExtIndex(self, 
				id,
				host='',
				port=None,
				fs_attribute='',
				record_separator='',
				REQUEST=None
				):
	
	""" egads... its the publisher string """
	
	self.addIndex(id, 'ExtIndex')

	ob = getattr(self, 'Indexes')
	indexes = ob.index_objects()
	for i in indexes:
		if i.id == id:
			break

	# how bout a catalog aware index:)
	i.manage_changeProperties(
						host=host,
						port=port, 
						fs_attribute=fs_attribute, 
						record_separator=record_separator
						)

	return REQUEST['RESPONSE'].redirect(
									REQUEST['URL3'] + '/manage_main?manage_tabs_message=Index%20Added')

	
class ExtIndex(PluggableIndex.PluggableIndex, Persistent,
	       Implicit, SimpleItem, PropertyManager):
	
	""" An ExtIndex provides an interface to searching
	and updating an external search engine against
	documents on the file system """
	
	__implements__ = (PluggableIndex.PluggableIndexInterface,)
	
	meta_type="ExtIndex"
	
	manage_options = (
		{'label':'Settings', 
		 'action':'manage_propertiesForm',
		 'help':'ExtIndex.stx'},
		   )
		
	_properties = (
		{ 'id':'host', 'type':'string', 'default':'localhost'},
		{ 'id':'port', 'type':'int', 'default':1967},
		{ 'id':'fs_attribute', 'type':'string', 'default':''},                   
		{ 'id':'record_separator', 'type':'string', 'default':''}               
		)
	
	query_options  = ["query"]
	
	def __init__(self, id, caller=None): 
		self.id = id
		self.clear()
	
	def getActiveFsObjs(self):
		""" return all fs paths that are active in the index.
		mainly intended for cron updating of actual external
		indexes"""
		
		return self._index.keys()
			
	# Pluggable Interface Below

	def numObjects(self):
		"""
		look an undocumented interface, note this is not used
	    as an interface in zope cvs 9/16/01
		"""
		return len(self._index.keys())	
	
	def clear(self):
		""" clear everything """
		self._index = OIBTree()
		self._unindex = IOBTree()
	
	def getEntryForObject(self, documentId, default=None):
		"""Get all information contained for a specific
		object by documentId"""
		
		if self._unindex.has_key(documentId):
			return self._unindex[documentId]
		
	def index_object(self, documentId, obj, threshold=None):
		"""Index an object """
		
		if hasattr(obj, self.fs_attribute):
			fpath = getattr(obj, self.fs_attribute)
			self._unindex[documentId]=fpath
			self._index[fpath]=documentId
			return 1
		
		return 0
		
	def unindex_object(self, documentId):
		"""Remove the documentId from the index"""
		
		if self._unindex.has_key(documentId):
			fpath = self._unindex[documentId]
			del self._unindex[documentId]
			del self._index[fpath]


	def uniqueValues(self, name=None, withLengths=0):
		"""Returns the unique values for name.
		
		If 'withLengths' is true,
		returns a sequence of tuples of
		(value, length)

		fudging an interface.
		"""
		return tuple(self._index.keys())
		
	def _apply_index(self, request, cid=''):
		""" parse search,
		    call ext search with request vars,
		    translate using unindex back to integer ids
		    create IISet and return"""

		record = parseIndexRequest(request,self.id,self.query_options)
		if record.keys==None: return None
		
		query = record.keys[0]
		results = swish_search(
						self.host, 
						self.port, 
						query=query
						)

		# does swish allow different record mappings?? only xml AFAIK
		filepathLambda = lambda x,y=self.record_separator: x.split(y)[1]		
		files = map(filepathLambda, results)

		#######################
		# you can comment this out if your zope indexes == swish indexes
		# in terms of content.
		has_file = self._index.has_key
		files = filter(has_file, files)
		#######################
		
		get_doc = self._index.__getitem__
		doc_ids = map(get_doc, files)
		
		return (IISet(doc_ids), (self.fs_attribute,))
	
	index_html=None
	manage_workspace = DTMLFile('dtml/manage_ExtIndex', globals())



